Attribute VB_Name = "basKeystrokes"
Option Explicit

Public Declare Function GetAsyncKeyState Lib "user32" (ByVal vKey As Long) As Integer
Public Declare Function GetKeyState Lib "user32" (ByVal nVirtKey As Long) As Integer

Public Const KEY_IS_PRESSED = -32767

Public Function GetCapslock() As Boolean
    GetCapslock = CBool(GetKeyState(vbKeyCapital) And 1)
End Function
Public Function GetNumLock() As Boolean
    GetNumLock = CBool(GetKeyState(vbKeyNumlock) And 1)
End Function
Public Function GetShift() As Boolean
    GetShift = CBool(GetAsyncKeyState(vbKeyShift))
End Function
Public Function GetControl() As Boolean
    GetControl = CBool(GetAsyncKeyState(vbKeyControl))
End Function

Public Function GetKeyStroke(Optional ReturnLiteral As Boolean = False, Optional IncludeMouseClicks As Boolean = False) As String
    
    'This function will intercept keystrokes made in any application (not necessarily the one
    'you are calling the function from). If ReturnLiteral is set to True, the function will return
    'actual valid ascii characters (eg. vbTab). Otherwise it will return a reference to the key pressed (eg. "[Tab]").
    'The IncludeMouseClicks switch will also pass a string whenever a mouse button is clicked.
    
    On Error Resume Next
    
    Dim KeyResult
    Dim KeySequencePos
    
    GetKeyStroke = ""
    
    '######################
    'Check for special keys
    '######################
    KeyResult = GetAsyncKeyState(vbKeyLButton)
    If KeyResult = KEY_IS_PRESSED And IncludeMouseClicks = True Then
        GetKeyStroke = "[Left Mouse Button]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyRButton)
    If KeyResult = KEY_IS_PRESSED And IncludeMouseClicks = True Then
        GetKeyStroke = "[Right Mouse Button]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyCancel)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Cancel]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyMButton)
    If KeyResult = KEY_IS_PRESSED And IncludeMouseClicks = True Then
        GetKeyStroke = "[Middle Mouse Button]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyBack)
    If KeyResult = KEY_IS_PRESSED Then
        If ReturnLiteral = True Then GetKeyStroke = vbBack Else: GetKeyStroke = "[Backspace]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyBack)
    If KeyResult = KEY_IS_PRESSED Then
        If ReturnLiteral = True Then GetKeyStroke = vbTab Else: GetKeyStroke = "[Tab]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyClear)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Clear]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyReturn)
    If KeyResult = KEY_IS_PRESSED Then
        If ReturnLiteral = True Then GetKeyStroke = vbCrLf Else: GetKeyStroke = "[Return]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyEscape)
    If KeyResult = KEY_IS_PRESSED Then
        If ReturnLiteral = True Then GetKeyStroke = Chr(27) Else: GetKeyStroke = "[Escape]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(32)
    If KeyResult = KEY_IS_PRESSED Then
        If ReturnLiteral = True Then GetKeyStroke = Chr(32) Else: GetKeyStroke = " "
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(33)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Page Up]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(34)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Page Down]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(35)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[End]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(36)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Home]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(37)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Left Arrow]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(38)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Up Arrow]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(39)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Right Arrow]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(40)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Down Arrow]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(41)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Select]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(42)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Print Screen]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(45)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Insert]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(46)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Delete]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(47)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "[Help]"
        Exit Function
    End If

    '##################################
    'Check for function keys
    '##################################
    KeyResult = GetAsyncKeyState(vbKeyF1)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F1"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF2)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F2"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF3)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F3"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF4)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F4"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF5)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F5"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF6)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F6"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF7)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F7"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF8)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F8"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF9)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F9"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF10)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F10"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF11)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F11"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF12)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F12"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF13)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F13"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF14)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F14"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF15)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F15"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyF16)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = "F16"
        Exit Function
    End If
    
    '##################################
    'Check for operator keys
    '##################################
    KeyResult = GetAsyncKeyState(2)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "/" Else GetKeyStroke = "?"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(186)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = ";" Else GetKeyStroke = ":"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(187)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "=" Else GetKeyStroke = "+"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(188)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "," Else GetKeyStroke = "<"
       Exit Function
    End If
    KeyResult = GetAsyncKeyState(189)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "-" Else GetKeyStroke = "_"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(190)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "." Else GetKeyStroke = ">"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(191)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "/" Else GetKeyStroke = "?"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(192)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "'" Else GetKeyStroke = "@"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(219)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "[" Else GetKeyStroke = "{"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(220)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "\" Else GetKeyStroke = "|"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(221)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "]" Else GetKeyStroke = "}"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(222)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "#" Else GetKeyStroke = "~"
        Exit Function
    End If

    '##################################
    'Check numbers on the main keyboard
    '##################################
    For KeySequencePos = 48 To 57
        KeyResult = GetAsyncKeyState(KeySequencePos)
        If KeyResult = KEY_IS_PRESSED Then
            If GetShift = True Or GetNumLock = True Then
                Select Case Val(Chr(KeySequencePos))
                    Case 1: GetKeyStroke = "!"
                    Case 2: GetKeyStroke = Chr(34)
                    Case 3: GetKeyStroke = ""
                    Case 4: GetKeyStroke = "$"
                    Case 5: GetKeyStroke = "%"
                    Case 6: GetKeyStroke = "^"
                    Case 7: GetKeyStroke = "&"
                    Case 8: GetKeyStroke = "*"
                    Case 9: GetKeyStroke = "("
                    Case 0: GetKeyStroke = ")"
                End Select
            Else
                GetKeyStroke = Chr(KeySequencePos)
            End If
            Exit Function
        End If
    Next KeySequencePos
    
    '###############################
    'Check for keys in the alphabet
    '###############################
    For KeySequencePos = 65 To 90
        KeyResult = GetAsyncKeyState(KeySequencePos)
        If KeyResult = KEY_IS_PRESSED Then
            If GetShift = False Then
                If GetCapslock = True Then GetKeyStroke = UCase(Chr(KeySequencePos)) Else GetKeyStroke = LCase(Chr(KeySequencePos))
            Else
                If GetCapslock = False Then GetKeyStroke = UCase(Chr(KeySequencePos)) Else GetKeyStroke = LCase(Chr(KeySequencePos))
            End If
            Exit Function
        End If
    Next KeySequencePos
    
    '###############################
    'Check numbers on the number pad
    '###############################
    KeyResult = GetAsyncKeyState(96)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "0" Else GetKeyStroke = ")"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(97)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "1" Else GetKeyStroke = "!"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(98)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "2" Else GetKeyStroke = Chr(34)
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(99)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "3" Else GetKeyStroke = "#"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(100)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "4" Else GetKeyStroke = "$"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(101)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "5" Else GetKeyStroke = "%"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(102)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "6" Else GetKeyStroke = "^"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(103)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "7" Else GetKeyStroke = "&"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(104)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "8" Else GetKeyStroke = "*"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(105)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "9" Else GetKeyStroke = "("
        Exit Function
    End If
        
    '##################################
    'Check other keys on the number pad
    '##################################
    KeyResult = GetAsyncKeyState(vbKeyMultiply)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "*"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyAdd)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "+"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeySeparator)
    If KeyResult = KEY_IS_PRESSED Then
        GetKeyStroke = ""
        If ReturnLiteral = True Then GetKeyStroke = vbCrLf Else: GetKeyStroke = "[Number Pad Enter]"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeySubtract)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "-"
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyDecimal)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "."
        Exit Function
    End If
    KeyResult = GetAsyncKeyState(vbKeyDivide)
    If KeyResult = KEY_IS_PRESSED Then
        If GetShift = False Then GetKeyStroke = "/"
        Exit Function
    End If
    
End Function
