/*########################################################################################################/
Typing Speedometer Widget for Konfabulator. Visually measures your typing speed in words-per-minute.

Copyright (C) 2005 William Warby

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General
Public License as published by the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write
to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
/########################################################################################################*/

/*########################################################################################################/
Version History:

1.5.2 (October 2005)
	- Added Windows .ico icon to package
	
1.5.1 (October 2005)
	- Fixed a bug in DLL registration where if the DLL was previously registered and then deleted an uncaught
	  exception would occur.
	- Added meta-tags for id, author, company, copyright and image

1.5.0 (August 2005)
	- Added COM support to enable the speedometer to work when the widget doesn't have the focus
	- Added support for CPM (characters per minute) mode
	- Added support for standard WPM measurement (where a word is equivalent to 5 keystrokes)
	- Added preference for varying the measurement period
	- Added reset count option from context menu

1.0.0 (August 2005)
	- Initial Release
/########################################################################################################*/

suppressUpdates();

//Hide type anywhere mode preference on non-Windows platforms
if (system.platform != 'windows') {
	preferences.typeAnywhere.hidden = true;
}

//Initialize variables
var intCount = 0;
var intCountMax = 0;
var intPrevCount = 0;
var intPrevCountMax = 0;
var intPinMax = 130;
var arrCount = new Array();
var arrCountMax = new Array();
var arrPrevCount = new Array();
var arrPrevCountMax = new Array();
var arrTypingTimes = new Array();
var arrCountStartOffset = new Array();
var arrCountShift = new Array();
var arrCountMaxStartOffset = new Array();
var arrCountMaxShift = new Array();
var blPrevSeparator = false;
var blDeletedPrevSeparator = false;
var intLastWordSeparator = 0;
var intReadoutChangeSteps = 0;
var intReadoutStep = 0;
var objSpeedometer = null;
var blCOMObjectCreated = false;

//Initialize images
imgTypingArea = new Image();
imgTypingArea.src = 'Resources/typing_background.png';

imgSpeedometer = new Image();
imgSpeedometer.src = 'Resources/background_wpm.png';
imgSpeedometer.hOffset = (imgTypingArea.width / 2) - (imgSpeedometer.width / 2);

imgTypingArea.vOffset = imgSpeedometer.height + 5;

imgReset = new Image();
imgReset.src = 'Resources/reset_count.png';
imgReset.hOffset = 18;
imgReset.vOffset = imgTypingArea.vOffset + imgTypingArea.height - imgReset.height - 15;
imgReset.onMouseDown = 'resetCount()';
imgReset.tracking = 'rectangle';

imgClear = new Image();
imgClear.src = 'Resources/clear.png';
imgClear.hOffset = imgTypingArea.width - imgClear.width - 18;
imgClear.vOffset = imgReset.vOffset;
imgClear.onMouseDown = 'txtTypingArea.data = \'Start typing here...\'';
imgClear.tracking = 'rectangle';

imgReadout = new Image();
imgReadout.src = 'Resources/readout.png';
imgReadout.hOffset = imgSpeedometer.hOffset + (imgSpeedometer.width / 2) - (imgReadout.width / 2);
imgReadout.vOffset = 130;

var intPushBack = (preferences.measurementType.value == 'CPM') ? 0 : 1;

imgReadoutDigits = new Array();
for (x = 0; x < 4; x++) {
	imgReadoutDigits[x] = new Image();
	if (preferences.measurementType.value != 'CPM' && x == 0) imgReadoutDigits[x].opacity = 0;
	if (system.platform == 'windows') {
		imgReadoutDigits[x].src = 'Resources/numbers_large.png';
		imgReadoutDigits[x].hOffset = imgReadout.hOffset + ((imgReadout.srcWidth / 3) * (x - intPushBack));
		imgReadoutDigits[x].vOffset = imgReadout.vOffset;
		imgReadoutDigits[x].clipRect = '0, 0, ' + imgReadoutDigits[x].width + ', ' + imgReadout.height;
	} else {
		imgReadoutDigits[x].src = 'Resources/0_large.png';
		imgReadoutDigits[x].hOffset = imgReadout.hOffset + ((imgReadout.srcWidth / 3) * (x - intPushBack));
		imgReadoutDigits[x].vOffset = imgReadout.vOffset;
	}
}

imgReadoutShadow = new Image();
imgReadoutShadow.src = 'Resources/readout_shadow.png';
imgReadoutShadow.hOffset = imgReadout.hOffset;
imgReadoutShadow.vOffset = imgReadout.vOffset;

imgReadoutMax = new Image();
imgReadoutMax.src = 'Resources/readout_max.png';
imgReadoutMax.hOffset = imgSpeedometer.hOffset + (imgSpeedometer.width / 2) - (imgReadoutMax.width / 2);
imgReadoutMax.vOffset = 114;

imgReadoutMaxDigits = new Array();
for (x = 0; x < 4; x++) {
	imgReadoutMaxDigits[x] = new Image();
	if (preferences.measurementType.value != 'CPM' && x == 0) imgReadoutMaxDigits[x].opacity = 0;
	if (system.platform == 'windows') {
		imgReadoutMaxDigits[x].src = 'Resources/numbers_small.png';
		imgReadoutMaxDigits[x].hOffset = imgReadoutMax.hOffset + ((imgReadoutMax.srcWidth / 3) * (x - intPushBack));
		imgReadoutMaxDigits[x].vOffset = imgReadoutMax.vOffset;
		imgReadoutMaxDigits[x].clipRect = '0, 0, ' + imgReadoutMaxDigits[x].width + ', ' + imgReadoutMax.height;
	} else {
		imgReadoutMaxDigits[x].src = 'Resources/0_small.png';
		imgReadoutMaxDigits[x].hOffset = imgReadoutMax.hOffset + ((imgReadoutMax.srcWidth / 3) * (x - intPushBack));
		imgReadoutMaxDigits[x].vOffset = imgReadoutMax.vOffset;
	}
}

imgReadoutMaxShadow = new Image();
imgReadoutMaxShadow.src = 'Resources/readout_max_shadow.png';
imgReadoutMaxShadow.hOffset = imgReadoutMax.hOffset;
imgReadoutMaxShadow.vOffset = imgReadoutMax.vOffset;

imgPin = new Image();
imgPin.src = 'Resources/pin.png';
imgPin.hRegistrationPoint = imgPin.width - 17;
imgPin.vRegistrationPoint = imgPin.height / 2;
imgPin.hOffset = imgSpeedometer.hOffset + (imgSpeedometer.width / 2);
imgPin.vOffset = imgSpeedometer.height / 2;
imgPin.rotation = 320;

imgPinHolder = new Image();
imgPinHolder.src = 'Resources/pin_holder.png';
imgPinHolder.hOffset = imgSpeedometer.hOffset + (imgSpeedometer.width / 2) - (imgPinHolder.width / 2);
imgPinHolder.vOffset = (imgSpeedometer.height / 2) - (imgPinHolder.height / 2);

imgGlaze = new Image();
imgGlaze.src = 'Resources/speedometer_glaze.png';
imgGlaze.hOffset = imgSpeedometer.hOffset;

//Initialize text area
txtTypingArea = new TextArea();
txtTypingArea.hOffset = 15;
txtTypingArea.vOffset = imgTypingArea.vOffset + 15;
txtTypingArea.width = imgTypingArea.width - 30;
txtTypingArea.height = imgClear.vOffset - imgTypingArea.vOffset - 18;
txtTypingArea.color = '#FFFFFF';
txtTypingArea.data = 'Start typing here...';
txtTypingArea.editable = true;
txtTypingArea.font = 'Arial, Helvetica, Sans-Serif';
txtTypingArea.size = 12;
txtTypingArea.onKeyPress = 'keyPressed();';
txtTypingArea.scrollbar = false;

tmrUpdate = new Timer();
tmrUpdate.interval = 0.5;
tmrUpdate.onTimerFired = 'updateDisplay();';
tmrUpdate.ticking = false;

tmrReadout = new Timer();
tmrReadout.interval = 0.05;
tmrReadout.onTimerFired = 'updateReadout();';
tmrReadout.ticking = false;

tmrIdle = new Timer()
tmrIdle.interval = 120;
tmrIdle.onTimerFired = 'tmrUpdate.ticking = false; tmrIdle.ticking = false;';
tmrIdle.ticking = false;

intReadoutChangeSteps = parseInt(tmrUpdate.interval / tmrReadout.interval) - 1;

setOptions();

updateNow();

function fitWindow() {
	if (imgTypingArea.opacity > 0) {
		wndMain.width = imgTypingArea.width;
		wndMain.height = imgTypingArea.vOffset + imgTypingArea.height;
	} else {
		wndMain.width = imgSpeedometer.hOffset + imgSpeedometer.width;
		wndMain.height = imgSpeedometer.vOffset + imgSpeedometer.height;
	}
}

function updateDisplay() {
	//Save current Count values
	intPrevCount = intCount;
	intPrevCountMax = intCountMax;
	
	//Get the time at the beginning of the measurement period
	var dateTemp = new Date();
	dateTemp.setTime(dateTemp.getTime() - (preferences.calculationPeriod.value * 1000));
	for (x = 0; x < arrTypingTimes.length; x++) {
		if (arrTypingTimes[x] < dateTemp) {
			arrTypingTimes.shift();
		} else {
			break;
		}
	}
	//Update new Count values
	intCount = arrTypingTimes.length;
	//If in WPM Standard mode, divide the number of characters typed by 5
	if (preferences.measurementType.value == 'WPM Standard') intCount = Math.floor(intCount / 5);
	
	//Adjust count for measurement period
	intCount = parseInt(intCount * (60 / preferences.calculationPeriod.value));
	
	if (intCount > intCountMax) intCountMax = intCount;
	//Rotate pin to the correct position
	var intPinCount = intCount;
	if (intPinCount > intPinMax) intPinCount = intPinMax;
	var intNewRotation;
	if (preferences.measurementType.value == 'CPM') {
		intNewRotation = intPinCount / 5;
	} else {
		intNewRotation = intPinCount * 2;
	}
	intNewRotation -= 40;
	if (intNewRotation < 0) intNewRotation += 360;
	imgPin.rotation = intNewRotation;
	
	//Create arrays of Count digits
	arrCount = new Array();
	for (x = 0; x < 4; x++) {
		if (intCount.toString().length > x) arrCount.push(parseInt(intCount.toString().substr(x, 1)));
		else arrCount.unshift(0);
	}
	arrCountMax = new Array();
	for (x = 0; x < 4; x++) {
		if (intCountMax.toString().length > x) arrCountMax.push(parseInt(intCountMax.toString().substr(x, 1)));
		else arrCountMax.unshift(0);
	}
	arrPrevCount = new Array();
	for (x = 0; x < 4; x++) {
		if (intPrevCount.toString().length > x) arrPrevCount.push(parseInt(intPrevCount.toString().substr(x, 1)));
		else arrPrevCount.unshift(0);
	}
	arrPrevCountMax = new Array();
	for (x = 0; x < 4; x++) {
		if (intPrevCountMax.toString().length > x) arrPrevCountMax.push(parseInt(intPrevCountMax.toString().substr(x, 1)));
		else arrPrevCountMax.unshift(0);
	}
	
	//Record current positions of readout dials
	for (x = 0; x < 4; x++) {
		arrCountStartOffset[x] = imgReadoutDigits[x].vOffset;
		arrCountMaxStartOffset[x] = imgReadoutMaxDigits[x].vOffset;
	}
	
	//Work out pixel shift amounts for each readout dial
	for (x = 0; x < 4; x++) {
		if (intPrevCount < intCount) {
			if (arrCount[x] > arrPrevCount[x]) {
				arrCountShift[x] = 0 - ((arrCount[x] - arrPrevCount[x]) * imgReadout.height);
			} else if (arrCount[x] < arrPrevCount[x]) {
				arrCountShift[x] = 0 - ((10 - arrPrevCount[x] + arrCount[x]) * imgReadout.height);
			} else {
				arrCountShift[x] = 0;
			}
		} else if (intPrevCount > intCount) {
			if (arrCount[x] < arrPrevCount[x]) {
				arrCountShift[x] = (arrPrevCount[x] - arrCount[x]) * imgReadout.height;
			} else if (arrCount[x] > arrPrevCount[x]) {
				arrCountShift[x] = ((10 - arrCount[x]) + arrPrevCount[x]) * imgReadout.height;
			} else {
				arrCountShift[x] = 0;
			}
		} else {
			arrCountShift[x] = 0;
		}
		if (intPrevCountMax < intCountMax) {
			if (arrCountMax[x] > arrPrevCountMax[x]) {
				arrCountMaxShift[x] = 0 - ((arrCountMax[x] - arrPrevCountMax[x]) * imgReadoutMax.height);
			} else if (arrCountMax[x] < arrPrevCountMax[x]) {
				arrCountMaxShift[x] = 0 - ((10 - arrPrevCountMax[x] + arrCountMax[x]) * imgReadoutMax.height);
			} else {
				arrCountMaxShift[x] = 0;
			}
		} else if (intPrevCountMax > intCountMax) {
			if (arrCountMax[x] < arrPrevCount[x]) {
				arrCountMaxShift[x] = (arrPrevCountMax[x] - arrCountMax[x]) * imgReadoutMax.height;
			} else if (arrCountMax[x] > arrPrevCountMax[x]) {
				arrCountMaxShift[x] = ((10 - arrCountMax[x]) + arrPrevCountMax[x]) * imgReadoutMax.height;
			} else {
				arrCountMaxShift[x] = 0;
			}
		} else {
			arrCountMaxShift[x] = 0;
		}
	}
	
	updateNow();
	
	//Start rotating readout dials
	intReadoutStep = 0;
	if (intPrevCount != intCount || intPrevCountMax != intCountMax) tmrReadout.ticking = true;
}

function updateReadout() {
	intReadoutStep++;
	for (x = 0; x < 4; x++) {
		if (system.platform == 'windows') {
			//Calculate new readout dial offset
			if (intReadoutStep < intReadoutChangeSteps) {
				intTempvOffset = parseInt(arrCountStartOffset[x] + (arrCountShift[x] * (intReadoutStep / intReadoutChangeSteps)));
			} else {
				intTempvOffset = arrCountStartOffset[x] + arrCountShift[x];
				intTempvOffset = imgReadout.vOffset - (arrCount[x] * imgReadout.height);
			}
			//Repstart readout dials if at either end of the dial
			if (intTempvOffset > imgReadout.vOffset) {
				intTempvOffset -= (imgReadoutDigits[x].height - imgReadout.height);
			} else if (intTempvOffset < imgReadout.vOffset - imgReadoutDigits[x].height + imgReadout.height) {
				intTempvOffset += (imgReadoutDigits[x].height - imgReadout.height);
			}
			//Reposition readout dial
			imgReadoutDigits[x].vOffset = intTempvOffset;
			imgReadoutDigits[x].clipRect = '0, ' + (imgReadout.vOffset - imgReadoutDigits[x].vOffset) + ', ' + imgReadoutDigits[x].width + ', ' + imgReadout.height;
		
			//Calculate new max readout dial offset
			if (intReadoutStep < intReadoutChangeSteps) {
				intTempvOffset = parseInt(arrCountMaxStartOffset[x] + (arrCountMaxShift[x] * (intReadoutStep / intReadoutChangeSteps)));
			} else {
				intTempvOffset = imgReadoutMax.vOffset - (arrCountMax[x] * imgReadoutMax.height);
			}
			//Repstart max readout dials if at either end of the dial
			if (intTempvOffset > imgReadout.vOffset) {
				intTempvOffset -= (imgReadoutMaxDigits[x].height - imgReadoutMax.height);
			} else if (intTempvOffset < imgReadoutMax.vOffset - imgReadoutMaxDigits[x].height + imgReadoutMax.height) {
				intTempvOffset += (imgReadoutMaxDigits[x].height - imgReadoutMax.height);
			}
			//Reposition max readout dial
			imgReadoutMaxDigits[x].vOffset = intTempvOffset;
			imgReadoutMaxDigits[x].clipRect = '0, ' + (imgReadoutMax.vOffset - imgReadoutMaxDigits[x].vOffset) + ', ' + imgReadoutMaxDigits[x].width + ', ' + imgReadoutMax.height;
		} else {
			if (intReadoutStep < parseInt(intReadoutChangeSteps / 2)) {
				if (arrCount[x] != arrPrevCount[x]) imgReadoutDigits[x].src = 'Resources/blur_large_' + random(1, 10) + '.png';
				if (arrCountMax[x] != arrPrevCountMax[x]) imgReadoutMaxDigits[x].src = 'Resources/blur_small_' + random(1, 10) + '.png';
			} else {
				imgReadoutDigits[x].src = 'Resources/' + arrCount[x] + '_large.png';
				imgReadoutMaxDigits[x].src = 'Resources/' + arrCountMax[x] + '_small.png';
			}
		}
	}
	
	updateNow();
	if (intReadoutStep >= intReadoutChangeSteps) tmrReadout.ticking = false;
}

function keyPressed(systemWideKey) {
	//Get the key which was last pressed
	var key;
	if (systemWideKey != null && systemWideKey != 'undefined' && systemWideKey != '') {
		key = systemWideKey;
	} else {
		key = system.event.keyString.toLowerCase();
	}
	if ((system.platform == 'windows' && key == 'backspace') || (system.platform == 'macintosh' && key == 'delete')) {
		//This key was backspace
		if (preferences.measurementType.value == 'WPM Actual Words') {
			if (intLastWordSeparator >= 1) {
				intLastWordSeparator--;
			} else if (!blDeletedPrevSeparator) {
				arrTypingTimes.pop();
				blPrevSeparator = false;
				blDeletedPrevSeparator = true;
			}
		} else {
			arrTypingTimes.pop();
		}
	} else if (((key == 'return' || key == 'space' || key == 'separator') && !blPrevSeparator) || preferences.measurementType.value == 'CPM' || preferences.measurementType.value == 'WPM Standard') {
		//This keypress was a word separator and the previous one wasn't (or we're in CPM mode) - add timestamp to array
		intLastWordSeparator = 0;
		blPrevSeparator = true;
		blDeletedPrevSeparator = false;
		var dateTemp = new Date();
		arrTypingTimes.push(dateTemp)
	} else {
		if (key != 'return' && key != 'space' && key != 'separator') {
			blPrevSeparator = false;
			blDeletedPrevSeparator = false;
			intLastWordSeparator++;
		}
	}
	
	//If update timer has been stopped due to inactivity, restart it.
	if (!tmrUpdate.ticking) tmrUpdate.ticking = true;
	
	//Restart the idle ticker to turn off updating after 120 seconds of inactivity
	tmrIdle.ticking = false;
	tmrIdle.ticking = true;
}

function resetCount() {
	
	//Stop the readout time from trying to update display
	tmrReadout.ticking = false;
	
	//Reset all variables
	intCount = 0;
	intCountMax = 0;
	intPrevCount = 0;
	intPrevCountMax = 0;
	arrCount = new Array();
	arrCountMax = new Array();
	arrPrevCount = new Array();
	arrPrevCountMax = new Array();
	arrTypingTimes = new Array();
	arrCountStartOffset = new Array();
	arrCountShift = new Array();
	arrCountMaxStartOffset = new Array();
	arrCountMaxShift = new Array();
	
	//Reset pin to starting rotation
	imgPin.rotation = -40;
	
	//Reset readout dials
	for (x = 0; x < 4; x++) {
		if (system.platform == 'windows') {
			imgReadoutDigits[x].vOffset = imgReadout.vOffset;
			imgReadoutDigits[x].clipRect = '0, 0, ' + imgReadoutDigits[x].width + ', ' + imgReadout.height;
			imgReadoutMaxDigits[x].vOffset = imgReadoutMax.vOffset;
			imgReadoutMaxDigits[x].clipRect = '0, 0, ' + imgReadoutMaxDigits[x].width + ', ' + imgReadoutMax.height;
		} else {
			imgReadoutDigits[x].src = 'Resources/0_large.png';
			imgReadoutMaxDigits[x].src = 'Resources/0_small.png';
		}
	}
}

function setOptions() {
	if (preferences.measurementType.value == 'CPM') {
		intPinMax = 1300;
		imgSpeedometer.src = 'Resources/background_cpm.png';
		imgReadout.width = imgReadout.srcWidth + (imgReadout.srcWidth / 3);
		imgReadout.vOffset = 125;
		imgReadoutMax.width = imgReadoutMax.srcWidth + (imgReadoutMax.srcWidth / 3);
		imgReadoutMax.vOffset = 109;
		imgReadoutDigits[0].opacity = 255;
		imgReadoutMaxDigits[0].opacity = 255;
	} else {
		intPinMax = 130;
		imgSpeedometer.src = 'Resources/background_wpm.png';
		imgReadout.width = imgReadout.srcWidth
		imgReadout.vOffset = 130;
		imgReadoutMax.width = imgReadoutMax.srcWidth
		imgReadoutMax.vOffset = 114;
		imgReadoutDigits[0].opacity = 0;
		imgReadoutMaxDigits[0].opacity = 0;
	}
	imgReadout.hOffset = imgSpeedometer.hOffset + (imgSpeedometer.width / 2) - (imgReadout.width / 2);
	imgReadoutShadow.width = imgReadout.width;
	imgReadoutShadow.hOffset = imgReadout.hOffset;
	imgReadoutShadow.vOffset = imgReadout.vOffset;
	imgReadoutMax.hOffset = imgSpeedometer.hOffset + (imgSpeedometer.width / 2) - (imgReadoutMax.width / 2);
	imgReadoutMaxShadow.width = imgReadoutMax.width;
	imgReadoutMaxShadow.hOffset = imgReadoutMax.hOffset;
	imgReadoutMaxShadow.vOffset = imgReadoutMax.vOffset;
	intPushBack = (preferences.measurementType.value == 'CPM') ? 0 : 1;
	for (x = 0; x < 4; x++) {
		imgReadoutDigits[x].hOffset = imgReadout.hOffset + ((imgReadout.srcWidth / 3) * (x - intPushBack));
		imgReadoutDigits[x].vOffset = imgReadout.vOffset;
		imgReadoutMaxDigits[x].hOffset = imgReadoutMax.hOffset + ((imgReadoutMax.srcWidth / 3) * (x - intPushBack));
		imgReadoutMaxDigits[x].vOffset = imgReadoutMax.vOffset;
	}
	if (preferences.typeAnywhere.value == true && system.platform == 'windows') {
		createCOMSpeedometer();
	}
	if (preferences.typeAnywhere.value == true && blCOMObjectCreated) {
		imgTypingArea.opacity = 0;
		imgReset.opacity = 0;
		imgClear.opacity = 0;
		txtTypingArea.opacity = 0;
	} else {
		destroyCOMSpeedometer();
		imgTypingArea.opacity = 255;
		imgReset.opacity = 255;
		imgClear.opacity = 255;
		txtTypingArea.opacity = 255;
	}
	fitWindow();
	updateNow();
}

function preferencesChanged() {
	resetCount();
	txtTypingArea.data = 'Start typing here...';
	setOptions();
}

function createCOMSpeedometer() {
	blError = '';
	if (objSpeedometer == null || objSpeedometer == 'undefined') {
		try {
			objSpeedometer = COM.createObject('TypingSpeedometer.Speedometer');
			if (objSpeedometer == null || objSpeedometer == 'undefined') {
				log('DLL not found. The file has probably been moved or deleted even though it was previously registered.');
				blError = 'notfound';
			}
		} catch (e) {
			log('Error creating COM object: ' + e);
			if (e.toLowerCase().indexOf('invalid class string') > -1) {
				blError = 'notinstalled';
			} else if (e.toLowerCase().indexOf('not be found') > -1) {
				blError = 'notfound';
			} else {
				blError = 'unknownerror';
			}
		}
	}
	
	if (blError != '') {
		//The DLL is not registered.
		if (blError == 'notinstalled') answer = alert('You have chosen to enable type anywhere mode for the first time. This works by installing a small DLL on your computer which tells the widget when a key is pressed in another window. IT DOES NOT tell the widget which key you pressed and cannot be used for malicious purposes, but you should only proceed if you are comfortable with this technique (see about box for more details of how it works). Would you like to continue and install the DLL?', 'Yes', 'No');
		else answer = 1;
		if (answer == 1) {
			log('Attempting to register DLL.');
			runCommand('./COMRegister.exe TypingSpeedometer.dll');
			blError = ''
			try {
				objSpeedometer = COM.createObject('TypingSpeedometer.Speedometer');
				if (objSpeedometer == null || objSpeedometer == 'undefined') {
					blError = 'stillnotfound';
					log('DLL still not found.');
				}
			} catch (e) {
				log('Error creating COM object for second attempt: ' + e);
				if (e.toLowerCase().indexOf('invalid class string') > -1) {
					blError = 'stillnotinstalled';
				} else {
					blError = 'stillunknownerror';
				}
			}
		}
	}

	if (blError != '') {
		//DLL register failed.
		if (blError == 'stillnotfound') {
			alert('The Widget was unable to find the Typing Speedometer DLL. Please report this to me through the widget gallery. For the meantime, the widget will revert to normal mode.');
		} else if (blError == 'stillnotinstalled') {
			alert('The Widget was unable to register the Typing Speedometer DLL. Please report this to me through the widget gallery. For the meantime, the widget will revert to normal mode.');
		} else {
			alert('An unknown error occurred whilst trying to initialize the Typing Speedometer DLL. Please report this to me through the widget gallery. For the meantime, the widget will revert to normal mode.');
		}
		preferences.typeAnywhere.value = false;
		blCOMObjectCreated = false;
		return false;
	} else {
		try {
			COM.connectObject(objSpeedometer, 'Speedometer_');
		} catch(e) {
			log('Error connecting COM object: ' + e);
			alert('An unknown error occurred whilst trying to connect to the Typing Speedometer DLL. Please report this to me through the widget gallery. For the meantime, the widget will revert to normal mode.');
			preferences.typeAnywhere.value = false;
			blCOMObjectCreated = false;
			return false;
		}
		blCOMObjectCreated = true;
		return true;
	}
}

function destroyCOMSpeedometer() {
	if (blCOMObjectCreated) COM.disconnectObject(objSpeedometer);
	delete objSpeedometer;
	blCOMObjectCreated = false;
}

function Speedometer_SystemWideKeyPress() {
	keyPressed(objSpeedometer.LastKeyType);
}